/**

	.NET rendering layer for ElevaSim simulator

	Jan Raszyk, 2006-2007

*/

#include "els_control.h"
#include <string>

using namespace System::Threading;

namespace elevasim {
	
	// starts a thread to simulate at max speed
	void StartTicking(void);

	// a loop that will run the simulation at maximum possible speed
	void KeepTicking(void);

	// constants determining simulation speed
	enum SimSpeed { SSPDR1 = 1, SSPDR2 = 2, SSPDR5 = 5, SSPDR10 = 10, SSPDMAX = 0 };

	enum dots_align;

	// stores user preferences adjusting the simulation process
	// wraps function that work with simulation controler
	class Renderer
	{

	private:
		Reader r;	// simulation controler
		enum { SIMSTOPPED = 0, SIMRUNNING } _state;
		int terminatesignal;
		SimSpeed _speed;
		std::string statusbuffer;

		// draws dots representing people in given location
		void DrawPeople(System::Drawing::Graphics ^g, int count, int, int, int, int, dots_align);

		// main drawing function
		void DrawBuilding(System::Drawing::Graphics ^g);

	public:
		int stop_at;	// time when stop the simulation

		Renderer(void);

		double seconds_elapsed; // counter
		void SetSpeed(SimSpeed s) { _speed = s; }
		SimSpeed GetSpeed(void) { return _speed; }
		
		// will send the simulation ticks until given ammount of miliseconds is elapsed
		void Elapse(int ms);

		// will shift the internal simulation clock for given ammount of seconds
		void SkipSeconds(int s);

		// reads simulation details from given file
		bool SimInit(System::String ^ filename);
		void SimClear(void);

		// write output stats into given file
		bool DumpStats(System::String ^ filename);

		bool SimRunning(void) { return _state == SIMRUNNING; }
		bool SimStart(void);	// user command
		bool SimRunFor(int);	// user command
		void SimStop(void);		// user command
		bool SimReady(void) { return r.SimReady(); }

		// adds more text into the status text buffer
		void StatusInsert(std::string);

		// gets all the text from the status text buffer
		System::String^ StatusBufferFlush(void);

		// returns a string with basic simulation info
		System::String^ SimDetails(void);
		
		// draws the current simulation state into the given PictureBox element
		void DrawSim(System::Windows::Forms::PictureBox ^);

		friend void KeepTicking(void);
		friend void StopTicking(void);
		friend void StartTicking(void);

	};

	// singleton class
	extern Renderer renderer;
}