/*

	ElevaSim simulation controler module

	Jan Raszyk, 2006-2007

*/
#pragma once
#include <string>

namespace elevasim {

	class Building;

	// interface for the simulation core
	// make an instantion of this class if you want to write your own rendering layer
	class Reader
	{
	private:
		bool _b_inited;
		Building* _b;
		int _b_id;

	public:
		Reader(void);

		// it makes sense to perform simulation steps if this is true
		bool SimReady(void);
		
		// read simulation specification from given file
		bool SimInit(std::string filename);

		// free simulation structures
		void SimClear(void);

		// dumps the statistics into a given file
		bool DumpStats(std::string filename);

		// perform simulation step
		bool Tick(void);

		// skip some time in the simulation
		void SkipSeconds(int);

		// elapsed simulation time
		double TimeElapsed(void);

		// height of the building
		double BuildingHeight(void);

		// number of floors in building
		int FloorsCount(void);

		// number of lifts in building
		int LiftsCount(void);

		// return altitude of given lift
		double LiftPosition(int);

		// returns the capacity of given lift
		int LiftCapacity(int);

		// returns altitude of given floor
		double FloorPosition(int);

		// returns number of prople in given lift
		int PeopleInLift(int);

		// returns number of people waiting for lift in upward direction
		int PeopleInFloorQU(int);

		// returns number of people waiting for lift in down direction
		int PeopleInFloorQD(int);

		// returns number of people idle in floor
		int PeopleInFloorR(int);
	};

}
