/*
	
	ElevaSim, Jan Raszyk, 2006-2007

	Human and associated classes are defined in here

*/

#include "els_common.h"

namespace elevasim {

	Human::Human()
	{
		duration_idle = 0; duration_travel = 0;
		type = HT_RESIDENT;
	}

	Human::Human(const Travel & f)
	{
		type = HT_VISITOR;
		my_flow = f;
		cur_stop = 0;
	}

	void Human::TravelTick(void) { duration_travel += TICK_INTERVAL; }

	void Human::WaitTick(void) { duration_idle += TICK_INTERVAL; }

	bool Human::FinalStop(void) 
	{
		if (type == HT_RESIDENT) return false;

		return cur_stop >= (int) my_flow.targets.size();
	}

	void HumanQueue::Tick(void)
	{
		for (QueueIter i = this->_queue.begin(); i != this->_queue.end(); i++)
			(*i)->WaitTick();
	}

	HumanPile::HumanPile(void)
	{
		// srand((unsigned) time(NULL));
	}

	void HumanQueue::IdleTick(Floor & f)
	{	// todo / fixme !
		QueueIter i;
		std::vector<int> to_del;
		Human *h;
		int floor_num = f.floor_num;
		int t_floor;

		for (i = this->_queue.begin(); i != _queue.end(); i++)
		{
			h = *i;
			if (h->type != HT_VISITOR) continue;
			
			h->cur_stop_time += TICK_INTERVAL;
			if (h->cur_stop_time > h->my_flow.time_spent[h->cur_stop]*60*60)
			{
				h->cur_stop++;
				f.visitors._queue.erase(i);
				t_floor = h->my_flow.targets[h->cur_stop];
				h->target_floor = t_floor;
				h->source_floor = floor_num;
				h->duration_idle = 0;
				h->duration_travel = 0;

				// call the lift for him
				f.blding->LiftCall(floor_num, t_floor);

				// place him into the queue
				if (t_floor > floor_num)
					f.lift_queue_up._queue.push_back(h);
				else
					f.lift_queue_down._queue.push_back(h);

			}
		}
	}


	void HumanPile::Push(Human* h)
	{
		this->_pile.push_back(h);
	}

	Human* HumanPile::PopRandom(void)
	/* picks a random member of the human pile */
	{
		int i = (rand() * (int) this->_pile.size()) / RAND_MAX;
		Human* h = this->_pile[i]; // this is a pointer!
		this->_pile[i] = this->_pile.back();
		this->_pile.pop_back();
		return h;
	}

}
